module newtri 
! Normal moveout
  use sep
  implicit none
  integer, private :: nt, i
  integer, private :: test 
  real, private :: t0, dt, xs, x, eps
  real, private :: dx, s02, wt, anti
  real, save, pointer, private :: sl(:)
  real, save, pointer, private :: g(:)
  contains

  subroutine newtri_init ( t0_in,dt_in,nt_in,x_in,dx_in, &
	slo,ga,s02_in,wt_in,anti_in, eps_in)

    real    :: t0_in, dt_in, x_in, eps_in
    real    :: dx_in, s02_in, wt_in, anti_in
    real, target    :: slo(:)
    real, target    :: ga(:)
    integer    :: nt_in 

    sl => slo
    g    => ga
    t0 = t0_in; dt = dt_in; nt = nt_in
    dx = dx_in; s02 = s02_in; wt = wt_in
    anti = anti_in; eps = eps_in
    x = x_in

  end subroutine newtri_init


  function newtri_lop ( adj,add, zz, tt) result(stat)

    integer            :: stat 
    logical,intent(in) :: adj,add
    real, dimension(:), intent(inout)  :: tt, zz
    call newtri_lop2(adj,add,zz,tt)
    stat=0

  end function newtri_lop


  subroutine newtri_lop2( adj,add,zz,tt )

    logical,intent(in)  :: adj,add 
    real, dimension(:), intent(inout)  :: zz, tt
    real, dimension(nt) :: ss
    integer   iz, itp, itm
    real t, z2, cc, xs2, g2, sg, t2, z, xs, cca
    real f, fx, gx
    real tm, tp, amp, slope
    
    ss = 0
    if (adj) zz=0
    if (adj) call douint(adj, add, ss, tt)

       do iz= 2, nt  

         z   = t0 + dt*(iz-1)
         z2  = z*z ;    xs = x*sl(iz)
         xs2 = xs*xs ;  g2 = g(iz)*g(iz)
         sg  = sqrt(g(iz))
         cc  = ( 2 - (g2 + 1/g2) )/( 4 * (sg + 1/sg) )
         if ((cc*xs2**2)<0. .and. &
	    (abs(cc*xs2**2)*z2/(z2 **2+eps**2) .ge. (z2 + xs2))) cycle
         t2  =  z2 + xs2  + ( (cc * xs2 * xs2) * z2 / (z2*z2 + eps**2) )
         t   = sqrt(t2)
         slope = anti * ( ((sl(iz)*sl(iz) - s02)*x ) +&
		(((sl(iz)*sl(iz))**2 - s02*s02)*x*x*x * (2*cc/z2)) )/(t+dt)

	 tp  = t + abs(slope*dx) + dt;  itp = 1.5 + (tp-t0)/dt
	 tm  = t - abs(slope*dx) - dt;  itm = 1.5 + (tm-t0)/dt

	 if (itm .le. 1 ) cycle
	 if (itp .ge. nt) exit

	 amp =wt * sqrt(nt*dt/(t+dt)) * (z+dt)/(t+dt) * (dt/(dt+tp-tm))**2

	 call spot( adj, .false.,  -amp, tm, zz(iz), ss)
	 call spot( adj, .false., 2*amp, t , zz(iz), ss)
	 call spot( adj, .false.,  -amp, tp, zz(iz), ss)

	end do

    if (.not. adj) call douint( adj, add, ss, tt)
	  
  end subroutine newtri_lop2

  subroutine douint(adj, add, pp, qq)
 
  logical,intent(in)  :: adj,add
  real, dimension(nt), intent(inout) :: pp, qq
  real, dimension(nt)                :: temp
  
  call adjnulo(adj, add, pp, qq)
  
  if (adj) then
	call cint(adj,     add, temp, qq)   
        call cint(.false., add, temp, pp)
  else
	call cint(adj,    add, pp, temp)
	call cint(.true., add, qq, temp)
  end if
	
  end subroutine douint

  subroutine cint(adj, add, p, q)

  logical, intent(in)   :: adj, add
  real, dimension(nt), intent(inout) :: p, q   
  real, dimension(nt) :: te
  integer id
	
  call adjnulo( adj, add, p, q)
  if (adj) then
	te(nt) = q(nt)
	do id=nt, 2, -1
		te(id-1) = te(id) + q(id-1)
	end do
	
	do id=1, nt
		p(id)    = p(id)  + te(id)
	end do

  else
	te(1) = p(1)
	do id=2, nt
		te(id) = te(id-1) + p(id)
	end do
	do id=1, nt
		q(id)  = q(id) + te(id)
	end do
  end if

  end subroutine cint 
	
  subroutine spot(adj, add, scale, tsp, val, vec)

    logical,intent(in)  :: adj,add
    real, intent(in)    :: tsp, scale
    real, intent(inout) :: val
    real, dimension(nt), intent(inout)  :: vec
    integer it, itc
    real tc, frac

    tc = .5 +(tsp-t0)/dt; itc=tc; it=1+itc; frac=tc-itc

    if (1 <= it .and. it <=nt )  then

	if (adj) then
		val = val + ((1. - frac) * vec(it) + &
			frac*vec(it+1)  ) * scale
	else
		vec(it  ) = vec(it  ) + (1. - frac) * val * scale
		vec(it+1) = vec(it+1) +   frac      * val * scale
	end if

    else
	call seperr("spt on the edge")
    end if

  end subroutine spot
	
  subroutine adjnulo(adj, add, x, y)
  logical, intent(in)  :: adj, add
  real, dimension(nt)  ::   x, y

  if (.not. add) then
	if ( adj )     x = 0
	if (.not. adj) y = 0
  end if

  end subroutine adjnulo

  subroutine nonhyp_clear()

  write(0,*) "done nonhyp"

  end subroutine nonhyp_clear


end module 
